@@ -44,6 +44,7 @@ OPTS = [
                default='openstacklocal',
                help='Domain to use for building the hostnames'),
     cfg.StrOpt('dnsmasq_config_file',
+               default='',
                help='Override the default dnsmasq settings with this file'),
     cfg.StrOpt('dnsmasq_dns_server',
                help='Use another DNS server before any in /etc/resolv.conf.'),
@@ -247,8 +248,7 @@ class Dnsmasq(DhcpLocalProcess):
                         mode,
                         self.conf.dhcp_lease_time))
 
-        if self.conf.dnsmasq_config_file:
-            cmd.append('--conf-file=%s' % self.conf.dnsmasq_config_file)
+        cmd.append('--conf-file=%s' % self.conf.dnsmasq_config_file)
         if self.conf.dnsmasq_dns_server:
             cmd.append('--server=%s' % self.conf.dnsmasq_dns_server)
 
