@@ -50,7 +50,10 @@ core_opts = [
     cfg.IntOpt('max_subnet_host_routes', default=20),
     cfg.StrOpt('state_path', default='.'),
     cfg.IntOpt('dhcp_lease_duration', default=120),
-    cfg.BoolOpt('allow_overlapping_ips', default=False)
+    cfg.BoolOpt('allow_overlapping_ips', default=False),
+    cfg.StrOpt('control_exchange',
+               default='quantum',
+               help='AMQP exchange to connect to if using RabbitMQ or Qpid')
 ]
 
 # Register the configuration options
